<?php

namespace SRVX\Service\Pay;


use Alipay\EasySDK\Kernel\Config;
use Alipay\EasySDK\Kernel\Factory;
use Alipay\EasySDK\Kernel\Util\ResponseChecker;
use SRVX\Msg;
use SRVX\Service\BaseService;


class EasyAlipayService extends BaseService
{
    protected $config = [];
    public $protocol = 'https';
    public $gatewayHost = 'openapi.alipay.com';
    public $signType = 'RSA2';

    public function __construct($config = [])
    {
        $this->getConfig($config);
        Factory::setOptions($this->setOptions());
    }

    private function setOptions()
    {
        $options                     = new Config();
        $options->protocol           = $this->protocol;
        $options->gatewayHost        = $this->gatewayHost;
        $options->signType           = $this->signType;
        $options->appId              = $this->config['pay_ali_app_id'];
        $options->alipayPublicKey    = $this->config['pay_ali_private_key'];
        $options->merchantPrivateKey = $this->config['pay_ali_public_key'];
        $options->notifyUrl          = $this->config['ali_notify_url'] ?? "";
        return $options;
    }

    private function getConfig($config)
    {
        $this->config = $config;
    }

    /**
     * app支付接口2.0
     * @param $out_trade_no string 商户网站唯一订单号。由商家自定义，64个字符以内，仅支持字母、数字、下划线且需保证在商户端不重复。
     * @param $total_amount string 订单总金额，单位为元，精确到小数点后两位，取值范围[0.01,100000000]，金额不能为0
     * @param $subject string 订单标题。注意：不可使用特殊字符，如 /，=，& 等。
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $optional array 可选业务参数（biz_content下的可选字段）的设置 (ps:每个参数由key和value组成，key和value的格式)
     */
    public function tradeAppPay($out_trade_no, $total_amount, $subject, $optional, $app_auth_token = "")
    {
        $response = (empty($optional)) ? Factory::payment()->app()->agent($app_auth_token)->pay($subject, $out_trade_no, $total_amount)
            : Factory::payment()->app()->agent($app_auth_token)->batchOptional($optional)->pay($subject, $out_trade_no, $total_amount);
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功
            return $this->returnRes(true, "success", $response->body);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    /**
     * alipay.trade.create(统一收单交易创建接口) 一码多付场景或小程序支付场景
     * @param $subject string 订单标题。注意：不可使用特殊字符，如 /，=，& 等。
     * @param $out_trade_no string 商户网站唯一订单号。由商家自定义，64个字符以内，仅支持字母、数字、下划线且需保证在商户端不重复。
     * @param $total_amount string 订单总金额，单位为元，精确到小数点后两位，取值范围[0.01,100000000]，金额不能为0
     * @param $buyer_id string 买家支付宝用户ID。2088开头的16位纯数字，小程序场景下获取用户ID请参考：用户授权;其它场景下获取用户ID请参考：网页授权获取用户信息;注：交易的买家与卖家不能相同。
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $optional array 可选业务参数（biz_content下的可选字段）的设置 (ps:每个参数由key和value组成，key和value的格式)
     * @throws \Exception
     */
    public function tradeCreate($subject, $out_trade_no, $total_amount, $buyer_id, $app_auth_token, $optional)
    {
        logger(json_encode($optional), "tradeCreate", true, true);
        $response = (empty($optional)) ? Factory::payment()->common()->agent($app_auth_token)->create($subject, $out_trade_no, $total_amount, $buyer_id)
                                       : Factory::payment()->common()->agent($app_auth_token)->batchOptional($optional)->create($subject, $out_trade_no, $total_amount, $buyer_id);
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功
            return $this->returnRes(true, $response->msg,  json_decode($response->httpBody, true)["alipay_trade_create_response"] ?? "");
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg),json_decode($response->httpBody, true)["alipay_trade_create_response"] ?? "");
        }
    }

    /**
     * alipay.trade.wap.pay(手机网站支付接口2.0)
     * @param $subject string 订单标题。注意：不可使用特殊字符，如 /，=，& 等。
     * @param $out_trade_no string 商户网站唯一订单号。由商家自定义，64个字符以内，仅支持字母、数字、下划线且需保证在商户端不重复。
     * @param $total_amount string 订单总金额，单位为元，精确到小数点后两位，取值范围[0.01,100000000]，金额不能为0
     * @param $buyer_id string 买家支付宝用户ID。2088开头的16位纯数字，小程序场景下获取用户ID请参考：用户授权;其它场景下获取用户ID请参考：网页授权获取用户信息;注：交易的买家与卖家不能相同。
     * @param $quit_url string 用户付款中途退出返回商户网站的地址
     * @param $return_url string 同步回调地址 todo::回调至第三方网站地址
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $optional array 可选业务参数（biz_content下的可选字段）的设置 (ps:每个参数由key和value组成，key和value的格式)
     * @return array
     * @throws \Exception
     */
    public function tradeWapPay($subject, $out_trade_no, $total_amount, $quit_url, $return_url, $app_auth_token, $optional)
    {
        $response = (empty($optional)) ? Factory::payment()->wap()->agent($app_auth_token)->pay($subject, $out_trade_no, $total_amount, $quit_url, $return_url)
            : Factory::payment()->wap()->agent($app_auth_token)->batchOptional($optional)->pay($subject, $out_trade_no, $total_amount, $quit_url, $return_url);
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功
            return $this->returnRes(true, $response->msg, $response->body);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    /**
     * alipay.trade.page.pay(统一收单下单并支付页面接口)
     * @param $subject string 订单标题。注意：不可使用特殊字符，如 /，=，& 等。
     * @param $out_trade_no string 商户网站唯一订单号。由商家自定义，64个字符以内，仅支持字母、数字、下划线且需保证在商户端不重复。
     * @param $total_amount string 订单总金额，单位为元，精确到小数点后两位，取值范围[0.01,100000000]，金额不能为0
     * @param $buyer_id string 买家支付宝用户ID。2088开头的16位纯数字，小程序场景下获取用户ID请参考：用户授权;其它场景下获取用户ID请参考：网页授权获取用户信息;注：交易的买家与卖家不能相同。
     * @param $return_url string 同步回调地址 todo::回调至第三方网站地址
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $optional array 可选业务参数（biz_content下的可选字段）的设置 (ps:每个参数由key和value组成，key和value的格式)
     * @return array
     * @throws \Exception
     */
    public function tradeWebPay($subject, $out_trade_no, $total_amount, $return_url, $app_auth_token = "", $optional = [], $httpmethod = "POST")
    {
        $this->logger([
            "subject"        => $subject,
            "out_trade_no"   => $out_trade_no,
            "total_amount"   => $total_amount,
            "return_url"     => $return_url,
            "app_auth_token" => $app_auth_token,
            "optional"       => $optional,
            "httpmethod"     => $httpmethod,
        ], 'webPayAction', true, true);
        $response = (empty($optional)) ? Factory::payment()->page()->agent($app_auth_token)
                                       : Factory::payment()->page()->agent($app_auth_token)->batchOptional($optional);
        $response = $response->pay($subject, $out_trade_no, $total_amount, $return_url, $httpmethod);
        $this->logger(["response" => $response], 'webPayAction', true);
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功
            return $this->returnRes(true, "success", $response->body);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }

    }

    /**
     * alipay.trade.pay(统一收单交易支付接口)  TODO::适用于 "扫用户出示的付款码，完成付款"
     * @param $subject string 订单标题。注意：不可使用特殊字符，如 /，=，& 等。
     * @param $out_trade_no string 商户网站唯一订单号。由商家自定义，64个字符以内，仅支持字母、数字、下划线且需保证在商户端不重复。
     * @param $total_amount string 订单总金额，单位为元，精确到小数点后两位，取值范围[0.01,100000000]，金额不能为0
     * @param $buyer_id string 买家支付宝用户ID。2088开头的16位纯数字，小程序场景下获取用户ID请参考：用户授权;其它场景下获取用户ID请参考：网页授权获取用户信息;注：交易的买家与卖家不能相同。
     * @param $authCode   string 支付授权码。
     * 当面付场景传买家的付款码（25~30开头的长度为16~24位的数字，实际字符串长度以开发者获取的付款码长度为准）或者刷脸标识串（fp开头的35位字符串）；
     * 周期扣款或代扣场景无需传入，协议号通过agreement_params参数传递；
     * 支付宝预授权和新当面资金授权场景无需传入，授权订单号通过 auth_no字段传入。
     * 注：交易的买家与卖家不能相同。
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $optional array 可选业务参数（biz_content下的可选字段）的设置 (ps:每个参数由key和value组成，key和value的格式)
     * @return array
     * @throws \Exception
     */
    public function tradeFaceToFaceForPay($subject, $out_trade_no, $total_amount, $buyer_id, $app_auth_token, $authCode, $optional)
    {
        $response = (empty($optional)) ? Factory::payment()->faceToFace()->agent($app_auth_token)->pay($subject, $out_trade_no, $total_amount, $authCode)
            : Factory::payment()->faceToFace()->agent($app_auth_token)->batchOptional($optional)->pay($subject, $out_trade_no, $total_amount, $authCode);
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功
            return $this->returnRes(true, $response->msg, $response->body);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    /**
     * alipay.trade.precreate(alipay.trade.precreate(统一收单线下交易预创建))  TODO::适用于 "生成交易付款码，待用户扫码付款"
     * @param $subject string 订单标题。注意：不可使用特殊字符，如 /，=，& 等。
     * @param $out_trade_no string 商户网站唯一订单号。由商家自定义，64个字符以内，仅支持字母、数字、下划线且需保证在商户端不重复。
     * @param $total_amount string 订单总金额，单位为元，精确到小数点后两位，取值范围[0.01,100000000]，金额不能为0
     * @param $buyer_id string 买家支付宝用户ID。2088开头的16位纯数字，小程序场景下获取用户ID请参考：用户授权;其它场景下获取用户ID请参考：网页授权获取用户信息;注：交易的买家与卖家不能相同。
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $optional array 可选业务参数（biz_content下的可选字段）的设置 (ps:每个参数由key和value组成，key和value的格式)
     * @return array
     * @throws \Exception
     */
    public function tradeFaceToFaceForPrecreate($subject, $out_trade_no, $total_amount, $app_auth_token, $optional)
    {
        $response = (empty($optional)) ? Factory::payment()->faceToFace()->agent($app_auth_token)->precreate($subject, $out_trade_no, $total_amount)
            : Factory::payment()->faceToFace()->agent($app_auth_token)->batchOptional($optional)->precreate($subject, $out_trade_no, $total_amount);

        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功
            return $this->returnRes(true, $response->msg, $response->body);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    /**
     * alipay.trade.query (统一收单线下交易查询) ps:该接口提供所有支付宝支付订单的查询，商户可以通过该接口主动查询订单状态，完成下一步的业务逻辑。
     * @param $out_trade_no string 订单支付时传入的商户订单号,和支付宝交易号不能同时为空。 trade_no,out_trade_no如果同时存在优先取trade_no
     * @param $trade_no string 支付宝交易号，和商户订单号不能同时为空
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     */
    public function tradeCommonQueryPay($out_trade_no = "", $trade_no = "", $app_auth_token)
    {
        if (empty($trade_no)) {
            $response = Factory::payment()->common()->agent($app_auth_token)->batchOptional(["trade_no" => $trade_no])->query($out_trade_no);
        } else {
            $response = Factory::payment()->common()->agent($app_auth_token)->query($out_trade_no);
        }
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功 todo::响应参数参见：https://opendocs.alipay.com/open/02c7hw
            return $this->returnRes(true, $response->msg, $response);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    /**
     * Todo::批量转账
     * @param $out_batch_no string 商户的批次号
     * @param $total_trans_amount string 批次总金额，单位为元，精确到小数点后两位，取值范围[0.01,9999999999999.99]
     * @param $total_count string 批次总笔数
     * @param $order_title  string 转账标题，用于收银台展示标题
     * @param $trans_order_list array 收款信息列表
     * Demo::
     * [
     * [
     * "out_biz_no" => "商户明细订单号",
     * "trans_amount" => "转账金额",
     * "payee_info" => [ //收款方信息
     * "identity" => "参与方的唯一标识(收款方的支付宝唯一ID )",
     * "identity_type" =>  "参与方的标识类型(1、ALIPAY_USER_ID 支付宝的会员ID 2、ALIPAY_LOGON_ID：支付宝登录号，支持邮箱和手机号格式)",
     * "name" =>  "参与方真实姓名，如果非空，将校验收款支付宝账号姓名一致性。当identity_type=ALIPAY_LOGON_ID时，本字段必填。"
     * ],
     * ]
     * ]
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $type string 1:web 2:app
     * @throws \Exception
     */
    public function tradeBatchTransfer($out_batch_no, $total_trans_amount, $total_count, $order_title, $trans_order_list, $passback_params, $app_auth_token, $type = "1")
    {
        //设置系统参数（OpenAPI中非biz_content里的参数）
        $textParams = array("app_auth_token" => $app_auth_token);
        //设置业务参数（OpenAPI中biz_content里的参数）
        $bizParams = [
            "out_batch_no"       => $out_batch_no,
            "product_code"       => "BATCH_API_TO_ACC",
            "biz_scene"          => "MESSAGE_BATCH_PAY",
            "order_title"        => $order_title,
            "total_trans_amount" => $total_trans_amount,
            "total_count"        => $total_count,
            "trans_order_list"   => $trans_order_list,
            "passback_params"    => $passback_params
        ];
        $response  = Factory::util()->generic()->execute("alipay.fund.batch.create", $textParams, $bizParams);
        logger(json_encode($response), "withdraw_response", true, true);
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            $batch_trans_id = json_decode($response->httpBody, true)["alipay_fund_batch_create_response"]["batch_trans_id"];
            return $this->returnRes(true, Msg::OpSuc, ["result" => $response->httpBody, "batch_trans_id" => $batch_trans_id]);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    /**
     * alipay.fund.trans.page.pay（pc端 - 批次支付接口）
     * @param $order_id  string 支付宝批次订单号。(alipay.fund.batch.create接口返回的batch_trans_id的值）
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param string $product_code 销售产品码 示例值：BATCH_API_TO_ACC
     * @param string $biz_scene 业务场景 示例值：MESSAGE_BATCH_PAY
     * @return array
     * @throws \Exception
     */
    public function tradeFundTransPagePay($order_id, $app_auth_token, $product_code = "BATCH_API_TO_ACC", $biz_scene = "MESSAGE_BATCH_PAY")
    {
        //设置系统参数（OpenAPI中非biz_content里的参数）
        $textParams = array("app_auth_token" => $app_auth_token);
        //设置业务参数（OpenAPI中biz_content里的参数）
        $bizParams = [
            "order_id"     => strval($order_id),
            "product_code" => $product_code,
            "biz_scene"    => $biz_scene,
        ];
        $response  = Factory::util()->generic()->execute("alipay.fund.trans.page.pay", $textParams, $bizParams);
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            //调用成功
            return $this->returnRes(true, $response->msg, $response->body);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    /**
     * alipay.system.oauth.token(换取授权访问令牌)
     * @param $code string 授权码，用户对应用授权后得到。本参数在 grant_type 为 authorization_code 时必填；为 refresh_token 时不填。
     * @param $app_auth_token string 刷新令牌，上次换取访问令牌时得到。本参数在 grant_type 为 authorization_code 时不填；为 refresh_token 时必填，且该值来源于此接口的返回值 app_refresh_token（即至少需要通过 grant_type=authorization_code 调用此接口一次才能获取）。
     * @param $grant_type string
         授权方式。支持：
        1.authorization_code，表示换取使用用户授权码code换取授权令牌access_token。
        2.refresh_token，表示使用refresh_token刷新获取新授权令牌。
     * @return array 响应示例
         {
            "alipay_system_oauth_token_response": {
            "user_id": "2088102150477652",  //支付宝用户的唯一标识。以2088开头的16位数字。
            "access_token": "20120823ac6ffaa4d2d84e7384bf983531473993", //访问令牌。通过该令牌调用需要授权类接口
            "expires_in": "3600", //访问令牌的有效时间，单位是秒。
            "refresh_token": "20120823ac6ffdsdf2d84e7384bf983531473993", //刷新令牌。通过该令牌可以刷新access_token
            "re_expires_in": "3600" //刷新令牌的有效时间，单位是秒。
            }
        }
     * @throws \Exception
     */
    public function systemOauthTokenOrRefreshToken($app_auth_token, $code = "", $refresh_token = "", $grant_type = "authorization_code")
    {
        $grant_type = !empty($code) ? $grant_type : "authorization_code";
        //设置系统参数（OpenAPI中非biz_content里的参数）
        $textParams = array("app_auth_token" => $app_auth_token,    "grant_type" => $grant_type, "code"       => $code,);
        //设置业务参数（OpenAPI中biz_content里的参数）
        if ($grant_type == "authorization_code") {
            $bizParams = [

            ];
            $response = Factory::util()->generic()->execute("alipay.system.oauth.token", $textParams, $bizParams);
            logger(json_encode(["response" => $response, "request" => ["textParams" => $textParams,]]), "system_oauth_token", true, true);
        } else {
            $bizParams = [
                "grant_type"    => $grant_type,
                "refresh_token" => $refresh_token,
            ];
            $response = Factory::util()->generic()->execute("alipay.system.oauth.token", $textParams, $bizParams);
            logger(json_encode(["response" => $response, "request" => ["textParams" => $textParams,]]), "system_oauth_refresh_token", true, true);
        }
        //处理响应或异常
        $responseChecker = new ResponseChecker();
        if ($responseChecker->success($response)) {
            return $this->returnRes(true, Msg::OpSuc, ["result" => $response->httpBody]);
        } else {
            //调用失败
            return $this->returnRes(false, ($response->subMsg ?? $response->msg), $response);
        }
    }

    public function verifyNotify($params)
    {
        return Factory::payment()->common()->verifyNotify($params);
    }

}